# -*- coding: utf-8 -*-
#
# Copyright 2010-2013 Kolab Systems AG (http://www.kolabsys.com)
#
# Jeroen van Meeuwen (Kolab Systems) <vanmeeuwen a kolabsys.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os, sys

from pykolab.introspection import *
import pykolab

from pykolab.constants import *
from pykolab.translate import _

log = pykolab.getLogger('pykolab.setup')
conf = pykolab.getConf()

components = {}
executed_components = []

components_included_in_cli = []

def __init__():
    import_commands(os.path.dirname(__file__), "pykolab.setup", "setup_", components)

    register('help', help, description=_("Display this help."))

def help(*args, **kw):
    list_commands(components, *args, **kw)
    print >> sys.stderr, _("\nSpecify 'all' to configure all components.")

def cli_options_from_component(component_name, *args, **kw):
    global components_included_in_cli

    if component_name in components_included_in_cli:
        return

    import_command_options(component_name, "component_name", "pykolab.setup", "setup_", components)
    components_included_in_cli.append(component_name)

def execute(component_name, *args, **kw):
    grouped_components = group_commands(components)
    sorted_components = get_sorted_commands(grouped_components)

    if not component_name:
        execute("help")
        return

    elif component_name == "all":
        while 1:
            for component in sorted_components:
                execute_this = True

                if component in executed_components:
                    execute_this = False

                if component == "help":
                    execute_this = False

                if execute_this:
                    if components[component].has_key('after'):
                        for _component in components[component]['after']:
                            if not _component in executed_components:
                                execute_this = False

                if execute_this:
                    execute(component)
                    executed_components.append(component)

            executed_all = True
            for component in sorted_components:
                if not component in executed_components and component != "help":
                    executed_all = False

            if executed_all:
                break

        return
    else:
        for component in sorted_components:
            cli_options_from_component(component)

    if not components.has_key(component_name):
        log.error(_("No such component."))
        sys.exit(1)

    if not components[component_name].has_key('function') and \
        not components[component_name].has_key('group'):
        log.error(_("No such component."))
        sys.exit(1)

    conf.finalize_conf()

    if len(conf.cli_args) >= 1:
        _component_name = conf.cli_args.pop(0)
    else:
        _component_name = component_name

    components[component_name]['function'](conf.cli_args, kw)

def register(*args, **kw):
    register_command(components, "component_name", *args, **kw)
