# -*- coding: utf-8 -*-
# Copyright 2010-2013 Kolab Systems AG (http://www.kolabsys.com)
#
# Jeroen van Meeuwen (Kolab Systems) <vanmeeuwen a kolabsys.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from ConfigParser import RawConfigParser
from os.path import join, isfile
from shutil import copy

import components

import pykolab

from pykolab import utils
from pykolab.constants import *
from pykolab.translate import _

log = pykolab.getLogger('pykolab.setup')
conf = pykolab.getConf()

def __init__():
    components.register(
            'freebusy',
            execute,
            description=description(),
            after=['ldap']
        )

def description():
    return _("Setup Free/Busy.")

def execute(*args, **kw):

    prefix = "/etc/kolab-freebusy"
    config = join(prefix, "config.ini")
    sample = join(prefix, "config.ini.sample")

    if not isfile(config) and not isfile(sample):
        if conf.check_only:
            utils.setup_status("freebusy", _("not installed"))
        else:
            log.error(_("Free/Busy is not installed on this system"))
        return

    if not isfile(config):
        if conf.check_only:
            utils.setup_status("freebusy", _("needs setup"))
            return
        copy(sample, config)

    freebusy_settings = [
            ('directory "kolab-ldap"', {
                    'host': conf.get('ldap', 'ldap_uri'),
                    'base_dn': conf.get('ldap', 'base_dn'),
                    'bind_dn': conf.get('ldap', 'service_bind_dn'),
                    'bind_pw': conf.get('ldap', 'service_bind_pw'),
                    'fbsource': 'file:/var/lib/kolab-freebusy/%mail.ifb',
                }),
            ('httpauth', {})
        ]

    cfg_parser = RawConfigParser()
    cfg_parser.read(config)

    will_update = False

    for section, definitions in freebusy_settings:
        if not definitions:
            cfg_parser.remove_section(section)
            continue

        for key, value in definitions.items():
            current = cfg_parser.get(section, key)

            # ConfigParser does not handle quoting for certain consumers of
            # .ini files like PHP's parse_ini_file.
            # See also: https://bugs.php.net/bug.php?id=36045

            value = "=" in value and ('"%s"' % value) or value

            if current != value:
                cfg_parser.set(section, key, value)
                will_update = True

    if conf.check_only:
        utils.setup_status("freebusy", will_update and _("needs setup") or _("setup done"))
        return

    if will_update:
        fp = open(config, "w+")
        cfg_parser.write(fp)
        fp.close()
