# -*- coding: utf-8 -*-
# Copyright 2010-2013 Kolab Systems AG (http://www.kolabsys.com)
#
# Jeroen van Meeuwen (Kolab Systems) <vanmeeuwen a kolabsys.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import components

import pykolab

from pykolab import utils
from pykolab.constants import *
from pykolab.setup.services import *
from pykolab.translate import _

log = pykolab.getLogger('pykolab.setup')
conf = pykolab.getConf()

def __init__():
    components.register(
            'kolabd',
            execute,
            description=description(),
            after=['ldap','imap']
        )

def description():
    return _("Setup the Kolab daemon.")

def execute(*args, **kw):

    # Test for the presence of the example.org template in order to determine if
    # work needs doing.

    if conf.has_section('example.org'):
        primary_domain = conf.get('kolab', 'primary_domain')

        if primary_domain != 'example.org':
            if conf.check_only:
                utils.setup_status("kolabd", _("needs setup"))
                return

            utils.multiline_message(
                    _("""
                            Copying the configuration section for 'example.org' over to
                            a section applicable to your domain '%s'.
                        """) % (primary_domain)
                )

            conf.cfg_parser._sections[primary_domain] = \
                    conf.cfg_parser._sections['example.org']
            conf.cfg_parser._sections.pop('example.org')

            fp = open(conf.cli_keywords.config_file, "w+")
            conf.cfg_parser.write(fp)
            fp.close()

    # Any absence of the example.org template indicates previous configuration,
    # and any continued presence not handled above indicates its use.

    if conf.check_only:
        utils.setup_status("kolabd", _("setup done"))
        return

    # Configure and manipulate the service.

    set_service_default('kolab-server', 'START', 'yes')

    service = is_debian() and 'kolab-server' or 'kolabd'
    if not control_service(service, 'restart'):
        log.error(_("Could not start the kolab server service."))

    if not configure_service(service, True):
        log.error(_("Could not configure to start on boot, the " + \
                "kolab server service."))
