# -*- coding: utf-8 -*-
# Copyright 2010-2013 Kolab Systems AG (http://www.kolabsys.com)
#
# Jeroen van Meeuwen (Kolab Systems) <vanmeeuwen a kolabsys.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from augeas import Augeas
from os.path import isfile, join

import components

import pykolab

from pykolab import utils
from pykolab.auth import Auth
from pykolab.constants import *
from pykolab.setup.services import *
from pykolab.translate import _

log = pykolab.getLogger('pykolab.setup')
conf = pykolab.getConf()

def __init__():
    components.register('php', execute, description=description())

def cli_options():
    php_group = conf.add_cli_parser_option_group(_("PHP Options"))

    php_group.add_option(
            "--timezone",
            dest    = "timezone",
            action  = "store",
            default = None,
            help    = _("Specify the timezone for PHP.")
        )

    php_group.add_option(
            "--with-php-ini",
            dest    = "php_ini_path",
            action  = "store",
            default = None,
            help    = _("Specify the path to the php.ini file used with the webserver.")
        )

    php_group.add_option(
            "--reset-php-config",
            dest    = "reset_php_config",
            action  = "store_true",
            default = False,
            help    = _("Reset the PHP configuration.")
        )

def description():
    return _("Setup PHP.")

def execute(*args, **kw):

    # Signal that interaction may occur. This will involve debconf and similar
    # system-specific mechanisms if available.

    start_interaction("kolab-conf/title-php")
    try:
        _execute(*args, **kw)
    finally:
        stop_interaction()

def _execute(*args, **kw):
    timezone = get_system_timezone()
    php_ini = get_php_ini_file()

    if not php_ini:
        if conf.php_ini_path:
            log.error(_("Cannot configure PHP through %r (No such file or directory)") % (conf.php_ini_path))
        else:
            log.error(_("Could not find PHP configuration file php.ini"))
        return

    myaugeas = Augeas()

    setting_base = '/files%s/' % php_ini

    setting = join(setting_base, 'Date', 'date.timezone')
    php_timezone = myaugeas.get(setting)

    if php_timezone and not conf.reset_php_config:
        if conf.check_only:
            utils.setup_status("php", _("setup done"))
        else:
            print >> sys.stderr, _("The PHP configuration is already modified and will not be configured.")
    else:
        if conf.check_only:
            utils.setup_status("php", _("needs setup"))
            return

        if timezone:
            log.info(_("Using system timezone for PHP."))
        elif conf.timezone:
            log.info(_("Using configured timezone for PHP."))
        else:
            timezone = conf.timezone = ask_question("kolab-conf/php-timezone",
                    _("""
                            Please supply the region-based timezone PHP should be using.
                            You have to use a continent or 'country/city' locality name
                            like 'Europe/Berlin'. Conventional time zones such as CEST
                            are not acceptable.
                        """),
                    _("Timezone ID"),
                    default="UTC"
                )

        insert_paths = myaugeas.match('/files%s/Date/*' % php_ini)
        myaugeas.insert(insert_paths[-1], 'date.timezone', False)

        log.debug(_("Setting key %r to %r") % ('Date/date.timezone', timezone), level=8)
        myaugeas.set(setting, timezone)

        myaugeas.save()
