# -*- coding: utf-8 -*-
# Copyright 2010-2013 Kolab Systems AG (http://www.kolabsys.com)
#
# Jeroen van Meeuwen (Kolab Systems) <vanmeeuwen a kolabsys.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from glob import glob
from os.path import join
from subprocess import Popen, PIPE
import os
import time
import sys

import components

import pykolab

from pykolab import utils
from pykolab.constants import *
from pykolab.setup.services import *
from pykolab.translate import _

log = pykolab.getLogger('pykolab.setup')
conf = pykolab.getConf()

def __init__():
    components.register('syncroton', execute, description=description(), after=['mysql','ldap','roundcube'])

def description():
    return _("Setup Syncroton.")

def execute(*args, **kw):
    schema_files = set()

    for syncroton_dir in glob('/usr/share/doc/kolab-syncroton*'):
        for root, directories, filenames in os.walk(syncroton_dir):
            for filename in filenames:
                if filename.startswith('mysql.initial') and filename.endswith('.sql'):
                    schema_files.add(join(root,filename))
        break

    # Stop if MySQL is not actually installed.

    if not have_mysql():
        if conf.check_only:
            utils.setup_status("syncroton", _("needs setup"))
        else:
            log.error(_("MySQL not installed: cannot initialise Syncroton."))
        return

    defaults_file = get_mysql_defaults()

    if not isfile(defaults_file):
        if conf.check_only:
            utils.setup_status("syncroton", _("needs setup"))
            return
        make_mysql_defaults_file(defaults_file)

    # Test for the Roundcube database.

    if not have_mysql_database(defaults_file, 'roundcube'):
        if conf.check_only:
            utils.setup_status("syncroton", _("needs setup"))
        else:
            log.error(_("Roundcube database not created: cannot initialise Syncroton."))
        return

    # Test for the presence of the plugin in Roundcube.

    if have_mysql_data(defaults_file, "roundcube",
        "select * from system where name = 'syncroton-version'",
        "syncroton-version"):

        if conf.check_only:
            utils.setup_status("syncroton", _("setup done"))
            return
        else:
            print >> sys.stderr, _("Kolab database account already exists and will not be configured.")
            log.info(_("A user called %s already exists. Not creating another one.") % 'kolab')

    else:
        if conf.check_only:
            utils.setup_status("syncroton", _("needs setup"))
            return

        for schema_file in schema_files:
            p1 = Popen(['cat', schema_file], stdout=PIPE)
            p2 = Popen(['mysql', '--defaults-file=%s' % defaults_file, 'roundcube'], stdin=p1.stdout)
            p2.communicate()

        time.sleep(2)

        apache = is_debian() and 'apache2' or 'httpd'
        if not control_service(apache, 'restart'):
            log.error(_("Could not start the webserver server service."))

        if not configure_service(apache, True):
            log.error(_("Could not configure to start on boot, the " + \
                    "webserver server service."))
